using System;
using System.Collections.Generic;
using System.Text;
using EC2Client;
using AmazonWebServices;

namespace EC2CMD
{
    class Program
    {
        public static EC2Operation Service;
        public static EC2Response Response;
        private static String _operation;
        private static String[] _arguments;
        private static String _accessKey;
        private static String _secretKey;
        private static bool inited;

        static void Main(string[] args)
        {
            inited = true;
            if (args.Length == 0)
            {
                Console.WriteLine("insufficient arguments list. run ec2cmd.exe -help to obtain help");
                inited = false;
            }
            if (inited == true)
            {

                if (args.Length < 3 & args.Length > 0 & args[0] != "-help")
                {
                    Console.WriteLine("insufficient arguments list. run ec2cmd.exe -help to obtain help");

                }
                else
                {
                    //Console.WriteLine("**** Amazon EC2 command-line interface ****");
                    //Console.WriteLine("********* .NET 2.0 implementation *********");
                    //Console.WriteLine("********* 2007 cyberlight systems *********");
                    //Console.Write("Command line parsing ... ");

                    _accessKey = args[0];
                    _secretKey = args[1];
                    _operation = args[2];
                    Array.Resize(ref _arguments, args.Length - 3); 

                    for (int i = 3; i < args.Length; i++)
                    {
                        _arguments[i - 3] = args[i];
                    }
                    //Console.Write("Done\r\n");
                    //Console.Write("Activating proxy ... ");
                    Service = new EC2Operation(_accessKey, _secretKey);

                    //Console.Write("Done\r\n");
                    //Console.Write("Performing XML-HTTP request ... ");
                    try
                    {
                        switch (_operation.ToUpper())
                        {
                            case "DESCRIBEIMAGES":
                                if (_arguments.Length > 0)
                                {
                                    Response = Service.DescribeImages(_arguments, null, null);
                                }
                                else
                                {
                                    Response = Service.DescribeImages(null, null, null);
                                }
                                break;

                            case "DESCRIBEINSTANCES":
                                if (_arguments.Length > 0)
                                {
                                    Response = Service.DescribeInstances(_arguments[0]);
                                }
                                else
                                {
                                    Response = Service.DescribeInstances(null);
                                }
                                break;

                            case "DESCRIBEKEYPAIRS":
                                if (_arguments.Length > 0)
                                {
                                    Response = Service.DescribeKeyPairs(_arguments[0]);
                                }
                                else
                                {
                                    Response = Service.DescribeKeyPairs(null);
                                }
                                break;

                            case "DESCRIBESECURITYGROUPS":
                                if (_arguments.Length > 0)
                                {
                                    Response = Service.DescribeSecurityGroups(_arguments[0]);
                                }
                                else
                                {
                                    Response = Service.DescribeSecurityGroups(null);
                                }
                                break;

                            case "DESCRIBEIMAGEATTRIBUTE":
                                Response = Service.DescribeImageAttribute(_arguments[0], _arguments[1]);
                                break;

                            case "MODIFYIMAGEATTRIBUTE":
                                Response = Service.ModifyImageAttribute(_arguments[0], _arguments[1], _arguments[2], _arguments[3], _arguments[4]);
                                break;

                            case "RESETIMAGEATTRIBUTE":
                                Response = Service.ResetImageAttribute(_arguments[0], _arguments[1]);
                                break;

                            case "REGISTERIMAGE":
                                Response = Service.RegisterImage(_arguments[0]);
                                break;

                            case "DEREGISTERIMAGE":
                                Response = Service.DeregisterImage(_arguments[0]);
                                break;

                            case "RUNINSTANCES":
                                Response = Service.RunInstances(_arguments[0], 1, int.Parse(_arguments[1]), _arguments[2], _arguments[3], _arguments[4]);
                                break;

                            case "TERMINATEINSTANCES":
                                Response = Service.TerminateInstances(_arguments[0]);
                                break;

                            case "REBOOTINSTANCES":
                                Response = Service.RebootInstances(_arguments[0]);
                                break;

                            case "GETCONSOLEOUTPUT":
                                Response = Service.GetConsoleOutput(_arguments[0]);
                                break;

                            case "CREATEKEYPAIR":
                                Response = Service.CreateKeyPair(_arguments[0]);
                                break;

                            case "DELETEKEYPAIR":
                                Response = Service.DeleteKeyPair(_arguments[0]);
                                break;

                            case "CREATESECURITYGROUP":
                                Response = Service.CreateSecurityGroup(_arguments[0], _arguments[1]);
                                break;

                            case "DELETESECURITYGROUP":
                                Response = Service.DeleteSecurityGroup(_arguments[0]);
                                break;

                            case "AUTHORIZESECURITYGROUP":
                                Response = Service.AuthorizeSecurityGroupIngress(_arguments[0], null, null, _arguments[1], _arguments[2], _arguments[3], _arguments[4]);
                                break;

                            case "REVOKESECURITYGROUP":
                                Response = Service.RevokeSecurityGroupIngress(_arguments[0], null, null, _arguments[1], _arguments[2], _arguments[3], _arguments[4]);
                                break;
                        }
                        //Console.Write("Done\r\n");
                        //Console.WriteLine(""); 
                        Console.Write(Response.Text);
                    }
                    catch (Exception e)
                    {
                        Console.Write("Failed\r\n");
                        Console.Write(e.Message);

                    }

                }

            }
            else 
            {
                Console.WriteLine("usage: ec2cmd.exe ACCESS-KEY SECRET-KEY operation [arguments]");
                Console.WriteLine("where:");
                Console.WriteLine("  ACCESS-KEY - AWS access key ID");
                Console.WriteLine("  SECRET-KEY - AWS secret key");
                Console.WriteLine("  operation  - one of following commands: ");
                Console.WriteLine("    DescribeImages [image-id1 image-id2 .... image-idN]");
                Console.WriteLine("    DescribeInstances [instance-id]");
                Console.WriteLine("    DescribeKeypairs [keyname]");
                Console.WriteLine("    DescribeSecurityGroups [group-name]");
                Console.WriteLine("    DescribeImageAttribute image-id attribute-name");
                Console.WriteLine("    ModifyImageAttribute image-id attribute-name attribute-operation user-id user-group");
                Console.WriteLine("    ResetImageAttribute image-id attribute-name");
                Console.WriteLine("    RegisterImage image-location");
                Console.WriteLine("    DeregisterImage image-id");
                Console.WriteLine("    RunInstances image-id max-count key-name user-data-string security-group-name");
                Console.WriteLine("    TerminateInstances instance-id");
                Console.WriteLine("    RebootInstances instance-id");
                Console.WriteLine("    GetConsoleOutput instance-id");
                Console.WriteLine("    CreateKeypair key-name");
                Console.WriteLine("    DeleteKeypair key-name");
                Console.WriteLine("    CreateSecurityGroup group-name group-desc");
                Console.WriteLine("    DeleteSecurityGroup group-name");
                Console.WriteLine("    AuthorizeSecurityGroup group-name TCP|UDP|ICMP src-port dest-port ip-range");
                Console.WriteLine("    RevokeSecurityGroup group-name TCP|UDP|ICMP src-port dest-port ip-range");
            }

        }
    }
}
