// This software code is made available "AS IS" without warranties of any 
// kind.  You may copy, display, modify and redistribute the software
// code either by itself or as incorporated into your code; provided that
// you do not remove any proprietary notices.  Your use of this software
// code is at your own risk and you waive any claim against Amazon Web 
// Services LLC or its affiliates with respect to your use of this software 
// code. (c) Amazon Web Services LLC or its affiliates.

using System;
using System.Collections.Generic;
using System.Text;
using System.Security.Policy;
using System.Web;
using System.Net;
using System.IO;
using System.Collections; 

namespace AmazonWebServices
{
	
	public class EC2Request
	{
	    
	    private IDictionary<String, String> _Parameters;
	    private String _SecretAccessKey;
	    private String _Host;
	    private String _Protocol;
	    
	    public EC2Request(string accessKeyId, string secretAccessKey, String signatureVersion, String version,
	                           String host, String protocol)
		{
		    _Parameters = new Dictionary<string,string>();
            _Parameters.Add("AWSAccessKeyId", accessKeyId);
            _Parameters.Add("SignatureVersion", signatureVersion);
		    _Parameters.Add("Version", version);
		    _Host = host;
		    _Protocol = protocol;
		    _SecretAccessKey = secretAccessKey;
		}
		
		// Adds a parameter to the parameter map
		public void AddParameter(String key, String value)
		{
		    if (value != null)
		    {
		        _Parameters.Add(key, value);
		    }
		}
		
		// Adds a list of parameters to the parameter map
		public void AddParameters(String key, IList<String> values)
		{
		    if (values != null && values.Count != 0)
		    {
    		    int i = 0;
		        foreach (String value in values)
		        {
		            ++i;
		            _Parameters.Add(key + "." + i.ToString(), value);
		        }
		    }
		}
		
		// Executes a specific action
		public EC2Response Execute(String action)
		{
		    // Add the action to the parameters list
		    _Parameters.Add("Action", action);
		    
		    // Add the current timestamp to the parameters
            string timestamp = EC2Helper.GetTimestamp();
		    _Parameters.Add("Timestamp", timestamp);
		    
		    // And the signature (based on the existing parameters)
            string signature = EC2Helper.GetSignature(_Parameters, _SecretAccessKey);
		    _Parameters.Add("Signature", signature);
		    
		    String uri = BuildUri();
		    //Console.WriteLine(uri);
		    return DoRequest(uri);
		}
		
		// Go and actually fetch the data from the EC2 
        private EC2Response DoRequest(string uri)
        {
            // Create the web request
            HttpWebRequest request = WebRequest.Create(uri) as HttpWebRequest;

            // Get response
            EC2Response ec2Response = null; 
            using (HttpWebResponse response = request.GetResponse() as HttpWebResponse)
            {
                ec2Response = new EC2Response(uri, response);
            }
            return ec2Response;
        }
        
        // Constructs a URI from the configured
		public String BuildUri()
		{
		    StringBuilder sb = new StringBuilder();
		    sb.Append(_Protocol);
		    sb.Append("://");
		    sb.Append(_Host);
		    sb.Append("/");
		    if (_Parameters.Count > 0)
		    {
		        sb.Append("?");
		        bool first = true;
    		    foreach (KeyValuePair<String, String> pair in _Parameters)
    		    {
    		        if (!first)
    		        {
    		            sb.Append("&");
    		        }
    		        else
    		        {
    		            first = false;
    		        }
    		        sb.Append(HttpUtility.UrlEncode(pair.Key));
    		        sb.Append("=");
    		        sb.Append(HttpUtility.UrlEncode(pair.Value));
                }
		    }
		    return sb.ToString();
		}
	}
}
